# -*- coding: utf8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Run the tunnel process and start a client, with a reactor as a fallback."""

import logging
import sys

from os import path

from twisted.internet import defer, reactor

from ubuntuone.platform.constants import TUNNEL_EXECUTABLE

logger = logging.getLogger("ubuntuone.SyncDaemon.TunnelRunner")


class TunnelRunner(object):
    """Run a tunnel process."""

    def __init__(self, host, port):
        """Start this runner instance."""
        self.client_d = defer.Deferred()
        try:
            self.start_process(host, port)
        except ImportError:
            logger.info("Proxy support not installed.")
            self.client_d.callback(reactor)

    def start_process(self, host, port):
        """Start the tunnel process."""
        from ubuntuone.proxy.tunnel_client import TunnelProcessProtocol
        protocol = TunnelProcessProtocol(self.client_d)
        process_path = self.get_process_path()
        args = [TUNNEL_EXECUTABLE, host, str(port)]
        reactor.spawnProcess(protocol, process_path, env=None, args=args)

    def get_process_path(self):
        """Get the path to the tunnel process."""
        # In a frozen setting, all binaries are in the same path
        if getattr(sys, "frozen", None) is not None:
            return path.join(path.dirname(
                path.abspath(sys.executable)), TUNNEL_EXECUTABLE)

        # This works when we are running from sources
        filename = path.join(path.dirname(__file__), "..", "..", "bin",
                             TUNNEL_EXECUTABLE)
        if path.isfile(filename):
            return filename

        # Use the configured LIBEXECDIR
        from ubuntuone.clientdefs import LIBEXECDIR
        return path.join(LIBEXECDIR, TUNNEL_EXECUTABLE)

    def get_client(self):
        """A deferred with the reactor or a tunnel client."""

        def client_selected(result, d):
            """The tunnel_client or the reactor were selected."""
            d.callback(result)
            # make sure the result is available for next callback
            return result

        d = defer.Deferred()
        self.client_d.addCallback(client_selected, d)
        return d
